clear all
est clear
graph close
graph drop _all
set type double
set seed 33333
set more off
set logtype text
capture log close
*this code is written for stata 14 and might need adjustments for other versions
*in particular, you must compile and run mata programs in the same or earlier version of stata 
version 14

/*
* to generate correlated binary x covariates install package rbinary 
net describe st0382, from("https://www.stata-journal.com/software/sj15-1/")
net install st0382, from("https://www.stata-journal.com/software/sj15-1/")
net get st0382, from("https://www.stata-journal.com/software/sj15-1/")
*ado uninstall st0382
*/

********************************************************************************************
* 1. Must provide the tax schedule as an input
********************************************************************************************
*this is ln(8) rounded to 4 sig digits
scalar kink =  2.0794

*rounded ln(1.3) and ln(0.9)
scalar s0 = 0.2624
scalar s1 = -0.1054

*the friction error only exists plus and minus deltap,deltam  around the kink
*rounded deltap(0.0953)=ln(1.1)=0.0953 and deltam(0.1054)=ln(0.9)=-0.1054 
*(note, although deltam is negative here, deltam is always positive as input to bunchfilter)
scalar deltam = -0.1054
scalar deltap = 0.0953

*frictions are from a truncated normal with sigma_e standard deviation
scalar sigma_e = 1

********************************************************************************************
* 2. generate fake data that looks like the actual data
********************************************************************************************
scalar N = 100000
* generate covariates for ability 
rbinary x1 x2 x3, means(0.2,0.5,0.3) corr(1,0.2,0.1\0.2,1,0.4\0.1,0.4,1) n(`=N')

*generate uniformly distributed weights that take the same structure at the IRS data
gen temp = uniform()
egen tot = total(temp)
sum tot
gen w = round((10^6)*temp/tot)
drop temp tot
sum w [fw=w]

* check covariates
sum x1 x2 x3 [fweight = w]
pwcorr x1 x2 x3 [fweight=w]

* from the tobit model
scalar beta0 = 2
scalar beta_x1 =  -0.2
scalar beta_x2 = 2.5
scalar beta_x3 = 0.4
scalar eps = 0.5
scalar sigma_nu = sqrt(0.5)

* generate observed ability
gen obs_nstar = `=beta0' + `=beta_x1'*x1 + `=beta_x2'*x2 + `=beta_x3'*x3
sum obs_nstar, d

*these should match SJ bunching.dta 
gen unobs_nstar = sigma_nu*rnormal()
sum unobs_nstar [fw=w], d

* generate ability
gen nstar= obs_nstar + unobs_nstar

* generate the friction error
gen u = runiform()
gen e = invnormal(normal(`=deltam' ) + u*(normal(`=deltap') - normal(`=deltam')))*`=sigma_e'

* generate taxable income according to the optimal reporting equation 
gen y = .
replace y = eps*s0+nstar if  nstar <  kink - eps*s0
replace y = kink    if (nstar >= kink - eps*s0)&(nstar <= kink - eps*s1)
replace y = eps*s1+nstar if  nstar >  kink - eps*s1 

gen yfric = y
replace yfric = y + e if y == `=kink'

keep y yfric x1 x2 x3 w

save bunching.dta, replace 

********************************************************************************************
* 3. check the help file example commands on this dataset 
********************************************************************************************
clear all
use bunching

bunchbounds y [fweight=w], k(2.0794) s0(0.2624) s1(-0.1054) m(2)

bunchtobit y x1 x2 x3, k(2.0794) s0(0.2624) s1(-0.1054) binwidth(0.08) 

bunchfilter yfric, generate(yfilter) kink(2.0794) deltam(0.1054) deltap(0.0953) pctobs(30) polorder(7)

bunching yfric x1 x2 x3, k(2.0794) s0(0.2624) s1(-0.1054) m(2) gen(yfilter2) deltam(0.1054) deltap(0.0953) pctobs(30) polorder(7) 

