*these are the demonstration commands in the help files

*AMP edits: 
* (1) edit inputs tax0(-.3) and tax1(.1) to s0(ln(1.3)) and s1(ln(.9)), using 4 decimal places. 
clear
set more off
capture graph close _all
capture log close
set logtype text
version 14
set scheme s2color 

********************************************************************************************
* 1. Preliminaries
********************************************************************************************
* set adodir
local adodir "G:\.shortcut-targets-by-id\1Ynnzj3tE2eVIWBHegh1IsHMx0i_AFWDd\McCallumSeegert\stata_journal\revision3\code\ado"
local thisdir "G:\.shortcut-targets-by-id\1Ynnzj3tE2eVIWBHegh1IsHMx0i_AFWDd\McCallumSeegert\stata_journal\revision3\code"
local outdir "G:\.shortcut-targets-by-id\1Ynnzj3tE2eVIWBHegh1IsHMx0i_AFWDd\McCallumSeegert\stata_journal\revision3\code\output"

*the rr version of the code and help files are here
sysdir set PERSONAL `adodir'

*you need to install the Stata package "lpdensity" before these estimates will work
*net install lpdensity, from(https://sites.google.com/site/nppackages/lpdensity/stata) replace

*uninstall old version if already installed
capture ado uninstall bunching

*ensure that you do not have bunching but do have lpdensity 
ado

*change to this directory
cd `thisdir'

*delete all prior output pdf files
cd `outdir'

!del *.pdf
!del *.txt
*look at the files in this directory and ensure we do not have any output
dir 

*the log file
log using reproduce_bunching_article_rr3.txt, replace

*change to this directory and load the simulated data
cd `adodir'
use bunching.dta, clear

*change working directory to output directory
cd `outdir'

********************************************************************************************
* 3.1 Results using simulated data 
********************************************************************************************
*this bin width will give you 100 bins
sum y [fw=w], d
scalar width =  round((r(max)-r(min))/100,0.001)
display `=width'

*table 1, covariates' proprieties
sum x1 x2 x3 [fw =w]
pwcorr x1 x2 x3 [fw=w]

*figure 1 combined histogram of simulated data, budget constraint and indifference curves
*generate histogram
twoway__histogram_gen y [fw=w], gen(den_y gridy) density width(0.084)
sort gridy

*set inputs

*the friction error only exists plus and minus deltap,deltam  around the kink
*rounded deltap(0.0953)=ln(1.1)=0.0953 and deltam(0.1054)=ln(0.9)=-0.1054 
*(note, although deltam is negative here, deltam is always positive as input to bunchfilter)
scalar deltam = -0.1054
scalar deltap = 0.0953

********************************************************************************************
* Figure 1 IC, budget constraints, and histogram
********************************************************************************************
*need true elasticity and tax schedule to create indifference curves
local eps = 0.5

*this is ln(8) rounded to 4 sig digits
local kink =  2.0794

*rounded ln(1.3) and ln(0.9)
local s0 = 0.2624
local s1 = -0.1054

*calculate intercept I0, level of utility at nunderline, IC for agent with nunderline
scalar I0 = 0
scalar Uunderline = I0 + [exp(`s0')-(exp(`eps'*`s0')^(1/`eps'))/(1+1/`eps')]*exp(`kink')
gen cunderline = log(Uunderline + [exp(`eps'*`s0'-`kink')^(1/`eps')/(1+1/`eps')]*exp(gridy)^(1+1/`eps'))
label variable cunderline "I.C. for n{superscript:{&lowast}}=k-{&epsilon}s{subscript:0} (right)"

*calculate intercept I1, level of utility at noverline, IC for agent with noverline
scalar I1 = I0  + exp(`s0')*exp(`kink')
scalar Uoverline = I0 + [exp(`s0')-(exp(`eps'*`s1')^(1/`eps'))/(1+1/`eps')]*exp(`kink')
gen coverline = log(Uoverline + [(exp(`eps'*`s1'-`kink')^(1/`eps'))/(1+1/`eps')]*exp(gridy)^(1+1/`eps'))
label variable coverline "I.C. for n{superscript:{&lowast}}=k-{&epsilon}s{subscript:1} (right)"

gen bc =log((gridy<=`kink')*(I0+exp(`s0')*exp(gridy))+ (gridy>`kink')*[I1+exp(`s1')*(exp(gridy)-exp(`kink'))])
label variable bc "Budget constraint (right)"

*graphing options
local xscalemin = -0.25
local xscalemax =  7.75

local xaxistitle = "Earnings (log thousands of $)"

local yscalemin = 0
local yscalemax = 5
local yaxistitle = "Consumption (log thousands of $)"

gen graphx = (gridy >= `xscalemin' & gridy <=`xscalemax')
gen graphbc = (bc >= `yscalemin' & bc <=`yscalemax')
gen graphcunder = (cunderline >= `yscalemin' & cunderline <=`yscalemax')
gen graphcover = (coverline >= `yscalemin' & coverline <=`yscalemax')

graph set window fontface "Times New Roman"

# delimit ; 
twoway (
bar den_y gridy if gridy >= -0.25 & gridy <= 7.75 , yaxis(1) bstyle(histogram) barwidth(0.084)  title("", ring(1) pos(11) size(medlarge)) 
)
(
 line bc gridy if graphx == 1 & graphbc == 1, yaxis(2) lwidth(thick) clcolor(black) clpattern(solid) 
)
(
 line cunderline gridy if graphx == 1 & graphcunder == 1,  yaxis(2) lwidth(thick) clcolor(black) clpattern(dot) 
)
(
 line coverline gridy if graphx == 1 & graphcover == 1, yaxis(2)  lwidth(thick) clcolor(black) clpattern(dash) 
)
(
pcarrowi 2.5 1.9 3.4 1, yaxis(2) lcolor(black) text(3.75 0.75 "Increasing" "utility (right)", yaxis(2)) barbsize(1) mcolor(black) 
)
,
title("", ring(1) pos(11) size(medlarge))
xscale(range(-0.25 7.75))
xlabel(0(2)8, labsize(large))
xtitle("Earnings (log thousands of $)", size(vlarge)) xline(2.0794)


yscale(range(0 1) axis(1))
ylabel(0(.2)1, axis(1) angle(horizontal) format(%12.2f) labsize(large) glwidth(thin) glcolor(black) glpattern(dot)) 
ytitle("Earnings density (100 bins)", axis(1) size(large))

yscale(range(0 5) axis(2))
ylabel(0(1)5, axis(2) angle(horizontal) labsize(large) glwidth(thin) glcolor(black) glpattern(dot)) 
ytitle(`yaxistitle', axis(2) size(large))

legend(ring(0) pos(3) label(1 "Data (left)") cols(1) order(1 2 3 4) symysize(*1) symxsize(*1) size(medium))
graphregion(margin(right) style(none) color(gs16));
# delimit cr

*set sj theme at top then use these edits for publication
gr_edit plotregion1.plot1.style.editstyle area(linestyle(color("%10"))) editcopy
gr_edit plotregion1.plot1.style.editstyle area(shadestyle(intensity(inten10))) editcopy

gr_edit plotregion1.AddTextBox added_text editor 0.95 1.1
gr_edit plotregion1.added_text[1].text = {}
gr_edit plotregion1.added_text[1].text.Arrpush Kink
gr_edit plotregion1.added_text[1].style.editstyle size(medium) editcopy
gr_edit plotregion2.textbox1.style.editstyle size(medium) editcopy

gr_edit plotregion1.plot1.style.editstyle area(shadestyle(color(black))) editcopy
gr_edit plotregion1.plot1.style.editstyle area(linestyle(color(black))) editcopy
gr_edit plotregion2._xylines[1].style.editstyle linestyle(color(black)) editcopy

graph export y.pdf, replace 

*clean up but keep bc and indifferenced curves
drop gridy den_y cunderline coverline bc graphx graphbc graphcunder graphcover
graph close

********************************************************************************************
* 3.2 Estimating elasticity bounds
********************************************************************************************
bunchbounds y [fw=w], k(2.0794) s0(0.2624) s1(-0.1054) m(2)
gr_edit .title.text = {}
gr_edit plotregion1.plot1.style.editstyle line(color(gs5)) editcopy
graph export bounds.pdf, name(bunchbounds) replace
graph close

bunchbounds y if x1 == 1 [fw=w], k(2.0794) s0(0.2624) s1(-0.1054) m(2)
gr_edit .title.text = {}
gr_edit plotregion1.plot1.style.editstyle line(color(gs5)) editcopy
graph export bounds_x1eq1.pdf, name(bunchbounds) replace
graph close

bunchbounds y if x1 == 0 [fw=w], k(2.0794) s0(0.2624) s1(-0.1054) m(2)
gr_edit .title.text = {}
gr_edit plotregion1.plot1.style.editstyle line(color(gs5)) editcopy
graph export bounds_x1eq0.pdf, name(bunchbounds) replace
graph close

bunchbounds y if x1 == 1 & x3 == 0 [fw=w], k(2.0794) s0(0.2624) s1(-0.1054) m(2)
gr_edit .title.text = {}
gr_edit plotregion1.plot1.style.editstyle line(color(gs5)) editcopy
graph export bounds_x1eq1_x3eq0.pdf, name(bunchbounds) replace
graph close

********************************************************************************************
* 3.3 Semi-parametric point estimates of the elasticity
********************************************************************************************
*correctly specified tobit model gives figure 3
bunchtobit y x1 x2 x3 [fw=w], k(2.0794) s0(0.2624) s1(-0.1054) binwidth(0.084) 

local distrlist "distr_100 distr_50 distr_20"

foreach i of local distrlist {
 graph rename bunchtobit_`i' correcttobit_`i', replace
 graph display correcttobit_`i'
 gr_edit .title.text = {}
 gr_edit plotregion1.plot1.style.editstyle area(linestyle(color("%10"))) editcopy
 gr_edit plotregion1.plot1.style.editstyle area(shadestyle(intensity(inten10))) editcopy
 
 gr_edit plotregion1.plot1.style.editstyle area(shadestyle(color(black))) editcopy
 gr_edit plotregion1.plot1.style.editstyle area(linestyle(color(black))) editcopy
 gr_edit plotregion1._xylines[1].style.editstyle linestyle(color(black)) editcopy
 
 if "`i'" == "distr_100" {
  gr_edit plotregion1.AddTextBox added_text editor 0.95 1.1
  gr_edit plotregion1.added_text[1].text = {}
  gr_edit plotregion1.added_text[1].text.Arrpush Kink
  gr_edit plotregion1.added_text[1].style.editstyle size(large) editcopy 
 }
 graph export correcttobit_`i'.pdf, name(correcttobit_`i') replace
}
graph rename bunchtobit_elast correcttobit_elast, replace
graph display correcttobit_elast
gr_edit .title.text = {}
gr_edit yaxis1.reset_rule 0 0.825 0.1 , tickset(major) ruletype(range) 
graph export correcttobit_elast.pdf, name(correcttobit_elast) replace
graph close 

*display output for the beta coefficients for the last subsample in the grid list
bunchtobit y x1 x2 x3 [fw=w], k(2.0794) s0(0.2624) s1(-0.1054) binwidth(0.084) grid(77)
estimates replay
graph close 

*incorrectly specified tobit model gives figure 4
bunchtobit y x1 x3 [fw=w], k(2.0794) s0(0.2624) s1(-0.1054) binwidth(0.084) 

foreach i of local distrlist {
 graph rename bunchtobit_`i' incorrecttobit_`i', replace
 graph display incorrecttobit_`i'
 gr_edit .title.text = {}
 gr_edit plotregion1.plot1.style.editstyle area(linestyle(color("%10"))) editcopy
 gr_edit plotregion1.plot1.style.editstyle area(shadestyle(intensity(inten10))) editcopy
 
 gr_edit plotregion1.plot1.style.editstyle area(shadestyle(color(black))) editcopy
 gr_edit plotregion1.plot1.style.editstyle area(linestyle(color(black))) editcopy
 gr_edit plotregion1._xylines[1].style.editstyle linestyle(color(black)) editcopy
 if "`i'" == "distr_100" {
  gr_edit plotregion1.AddTextBox added_text editor 0.95 1.1
  gr_edit plotregion1.added_text[1].text = {}
  gr_edit plotregion1.added_text[1].text.Arrpush Kink
  gr_edit plotregion1.added_text[1].style.editstyle size(large) editcopy
 }
 graph export incorrecttobit_`i'.pdf, name(incorrecttobit_`i') replace
}
graph rename bunchtobit_elast incorrecttobit_elast, replace
graph display incorrecttobit_elast
gr_edit .title.text = {}
graph export incorrecttobit_elast.pdf, name(incorrecttobit_elast) replace
graph close 

********************************************************************************************
* 4.2 Example for bunchfilter
********************************************************************************************
*filter the data and create figure 5
bunchfilter yfric [fw=w], kink(2.0794) deltam(0.12) deltap(0.12) generate(yfiltered) binwidth(0.084) pctobs(30) 

graph rename bunchfilter_two_cdfs filter_cdf, replace
graph display filter_cdf
gr_edit .title.text = {}
gr_edit plotregion1.plot2.style.editstyle marker(fillcolor(black)) editcopy
gr_edit plotregion1.plot2.style.editstyle marker(linestyle(color(black))) editcopy
gr_edit plotregion1.plot1.style.editstyle marker(fillcolor(gs10)) editcopy
gr_edit plotregion1.plot1.style.editstyle marker(linestyle(color(gs10))) editcopy
gr_edit plotregion1.plot1.style.editstyle marker(size(medium)) editcopy
gr_edit legend.plotregion1.key[1].view.style.editstyle marker(fillcolor(gs10)) editcopy
gr_edit legend.plotregion1.key[1].view.style.editstyle marker(linestyle(color(gs10))) editcopy
gr_edit plotregion1.AddTextBox added_text editor 0.37 2.1
gr_edit plotregion1.added_text[1].text = {}
gr_edit plotregion1.added_text[1].text.Arrpush Kink
gr_edit plotregion1.added_text[1].style.editstyle size(large) editcopy
gr_edit plotregion1._xylines[1].style.editstyle linestyle(color(black)) editcopy
graph export filter_cdf.pdf, name(filter_cdf) replace


graph rename bunchfilter_den_1 filter_pdf, replace
graph display filter_pdf
gr_edit .title.text = {}
gr_edit plotregion1.plot1.style.editstyle area(linestyle(color(black))) editcopy
gr_edit plotregion1.plot1.style.editstyle area(shadestyle(color(black))) editcopy
gr_edit plotregion1.plot2.style.editstyle area(linestyle(color("%10"))) editcopy
gr_edit plotregion1.plot2.style.editstyle area(shadestyle(intensity(inten10))) editcopy

gr_edit yaxis1.title.text = {}
gr_edit yaxis1.title.text.Arrpush `"Density (100 bins)"'
graph export filter_pdf.pdf, name(filter_pdf) replace
graph close 

*check actual bunching and filtered bunching
gen bunch = 100*(2.0794==y)
gen bunchfric = 100*(2.0794==yfric)
gen bunchfilter = 100*(2.0794==yfiltered)

format bunch* %9.2fc
sum bunch [fw=w]
sum bunchfric [fw=w]
sum bunchfilter [fw=w]

********************************************************************************************
* 5 Automatic estimation 
********************************************************************************************
*Automatic filtering, bounds, and semi-parametric estimation and figure 6
bunching yfric x1 x3, kink(2.0794) s0(0.2624) s1(-0.1054) m(2) binwidth(0.084) deltam(0.12) deltap(0.12) gen(ybunching) pctobs(30) 

graph rename bunchbounds fric_bounds, replace
graph display fric_bounds
gr_edit .title.text = {}
gr_edit plotregion1.plot1.style.editstyle line(color(gs5)) editcopy
graph export fric_bounds.pdf, name(fric_bounds) replace

local fricdistrlist "distr_40 distr_20"
foreach i of local fricdistrlist {
 graph rename bunchtobit_`i' fric_incorrecttobit_`i', replace
 graph display fric_incorrecttobit_`i'
 gr_edit .title.text = {}
 gr_edit .xaxis1.title.text = {}
 
 gr_edit plotregion1.plot1.style.editstyle area(linestyle(color("%10"))) editcopy
 gr_edit plotregion1.plot1.style.editstyle area(shadestyle(intensity(inten10))) editcopy
 
 gr_edit plotregion1.plot1.style.editstyle area(shadestyle(color(black))) editcopy
 gr_edit plotregion1.plot1.style.editstyle area(linestyle(color(black))) editcopy
 gr_edit plotregion1._xylines[1].style.editstyle linestyle(color(black)) editcopy
 
 graph export fric_incorrecttobit_`i'.pdf, name(fric_incorrecttobit_`i') replace
}
graph rename bunchtobit_elast fric_incorrecttobit_elast, replace
graph display fric_incorrecttobit_elast
gr_edit .title.text = {}
gr_edit .xaxis1.title.text = {}
graph export fric_incorrecttobit_elast.pdf, name(fric_incorrecttobit_elast) replace

capture log close 